import sys
import uproot
import pandas
import pickle
import xgboost as xgb
from xgboost.sklearn import XGBClassifier
from sklearn.model_selection import train_test_split
from sklearn.metrics import roc_curve
import matplotlib.pyplot as plt

# Besser: Einlesen mit argparse
trainfile = sys.argv[1]

train_data = uproot.open(trainfile)["DecayTree"].arrays(outputtype=pandas.DataFrame)

bdt = XGBClassifier(n_estimators=1000, max_depth=3, n_threads=10)

X, x, Y, y = train_test_split(train_data[["f1", "f2", "f3"]], train_data.label, test_size=0.4)

bdt.fit(X, Y, eval_set=[(X, Y), (x, y)], early_stopping_rounds=20)

# Save bdt to file
pickle.dump(bdt, open("bdt.model", "wb"))

# Done!

print("Feature importance for f1, f2, f3:")
print(bdt.feature_importances_)

print("Making performance plots")
# ROC curve
fpr, tpr, _ = roc_curve(y, bdt.predict_proba(x)[:,1])
plt.figure(figsize=(8, 6))
plt.plot(fpr, tpr)
plt.xlabel("False positive rate")
plt.ylabel("True positive rate")
plt.savefig("roc.pdf")

# Error rate during training
validation_score = bdt.evals_result()
iteration = range(len(validation_score["validation_0"]["error"]))
plt.figure(figsize=(8, 6))
plt.plot(iteration, validation_score["validation_0"]["error"], label="Training performance")
plt.plot(iteration, validation_score["validation_1"]["error"], label="Test performance")
plt.xlabel("iteration")
plt.ylabel("error rate")
plt.legend()
plt.savefig("train_performance.pdf")
